{-# LANGUAGE OverloadedStrings, StandaloneDeriving, TemplateHaskell  #-}
{-# OPTIONS_GHC -Wall #-}
module Text.LaTeX.Entities
    ( unicodeToLaTeX
    , unicodeLaTeXMap
    , isoentHeaders
    , latexEntities
    , latexEntitiesUnknown
    ) where

import Data.Char (chr)
import Data.FileEmbed (embedFile)
import qualified Data.Map as Map
import Data.Monoid ((<>))
import Data.Text as Text (Text)
import Data.Text.Encoding (decodeUtf8)
import Text.LaTeX as LTX (raw, usepackage)
import Text.LaTeX.Base.Class as LTX (LaTeXC)
import Text.LaTeX.Base.Syntax (LaTeX(..), MathType(..))

-- deriving instance Ord TeXArg
-- deriving instance Ord LaTeX

isoentSty :: Text
isoentSty = decodeUtf8 $(embedFile "isoent.sty")

-- |Return the (non-math) LaTeX sequence for some known unicode
-- characters.  At this time the standard ASCII characters are not
-- covered.
unicodeToLaTeX :: Char -> [LaTeX]
unicodeToLaTeX c = Map.findWithDefault [] c unicodeLaTeXMap

-- | A map from Unicode code point to a list of possible LaTeX
-- representations, with the first being preferred.
unicodeLaTeXMap :: Map.Map Char [LaTeX]
unicodeLaTeXMap =
    Map.fromListWith (++) (map (\ (uc, ltx) -> (uc, [ltx])) (latexEntities))

latexEntities :: [(Char, LaTeX)]
latexEntities =
    [ (chr 8195, raw "\\hspace{1em}")
    , (chr 8195, m (raw "\\quad"))
    , (chr 8194, raw "\\hspace{0.5em}")
    , (chr 8194, m (raw "\\;"))
    , (chr 8196, raw "\\hspace{0.33em}")
    , (chr 8196, m (raw "\\:"))
    , (chr 8197, raw "\\hspace{0.25em}")
    , (chr 8199, raw "\\hphantom{0}")
    , (chr 8200, raw "\\hphantom{,}")
    , (chr 8201, raw "\\hspace{0.167em}")
    , (chr 8201, m (raw "\\,"))
    , (chr 8202, raw "\\hspace{1pt}")
    , (chr 8212, raw "---")
    , (chr 8212, raw "\\textemdash")
    , (chr 8211, raw "--")
    , (chr 8211, raw "\\textendash")
    , (chr 8208, raw "-")
    , (chr 9251, raw "\\textvisiblespace")
    , (chr 8230, raw "\\ldots")
    , (chr 8230, raw "\\textellipsis")
    , (chr 8229, raw "\\nldr")
    , (chr 8531, m (raw "\\sfrac{1}{3}"))
    , (chr 8531, m (raw "\\frac{1}{3}"))
    , (chr 8532, m (raw "\\sfrac{2}{3}"))
    , (chr 8532, m (raw "\\frac{2}{3}"))
    , (chr 8533, m (raw "\\sfrac{1}{5}"))
    , (chr 8533, m (raw "\\frac{1}{5}"))
    , (chr 8534, m (raw "\\sfrac{2}{5}"))
    , (chr 8534, m (raw "\\frac{2}{5}"))
    , (chr 8535, m (raw "\\sfrac{3}{5}"))
    , (chr 8535, m (raw "\\frac{3}{5}"))
    , (chr 8536, m (raw "\\sfrac{4}{5}"))
    , (chr 8536, m (raw "\\frac{4}{5}"))
    , (chr 8537, m (raw "\\sfrac{1}{6}"))
    , (chr 8537, m (raw "\\frac{1}{6}"))
    , (chr 8538, m (raw "\\sfrac{5}{6}"))
    , (chr 8538, m (raw "\\frac{5}{6}"))
    , (chr 8453, m (raw "{^c\\!/\\!_o}"))
    , (chr 9608, raw "\\block")
    , (chr 9600, raw "\\uhblk")
    , (chr 9604, raw "\\lhblk")
    , (chr 9646, raw "\\marker")
    , (chr 9675, m (raw "\\circ"))
    , (chr 9633, m (raw "\\square"))
    , (chr 9645, raw "\\fbox{~~}")
    , (chr 9653, m (raw "\\vartriangle"))
    , (chr 9653, m (raw "\\triangle"))
    , (chr 9663, m (raw "\\triangledown"))
    , (chr 9734, raw "\\ding{73}")
    , (chr 8226, raw "\\textbullet")
    , (chr 8226, m (raw "\\bullet"))
    , (chr 9642, m (raw "\\blacksquare"))
    , (chr 9652, m (raw "\\blacktriangle"))
    , (chr 9662, m (raw "\\blacktriangledown"))
    , (chr 9666, m (raw "\\blacktriangleleft"))
    , (chr 9656, m (raw "\\blacktriangleright"))
    , (chr 9827, raw "\\ding{168}")
    , (chr 9827, m (raw "\\clubsuit"))
    , (chr 9830, raw "\\ding{169}")
    , (chr 9830, m (raw "\\diamondsuit"))
    , (chr 9829, raw "\\ding{170}")
    , (chr 9829, m (raw "\\heartsuit"))
    , (chr 9824, raw "\\ding{171}")
    , (chr 9824, m (raw "\\spadesuit"))
    , (chr 10016, raw "\\maltese")
    , (chr 10016, raw "\\ding{64}")
    , (chr 8224, raw "\\dag")
    , (chr 8224, raw "\\textdagger")
    , (chr 8224, m (raw "\\dagger"))
    , (chr 8225, raw "\\ddag")
    , (chr 8225, raw "\\textdaggerdbl")
    , (chr 8225, m (raw "\\ddagger"))
    , (chr 10003, raw "\\checkmark")
    , (chr 10003, raw "\\ding{51}")
    , (chr 10007, raw "\\ding{55}")
    , (chr 9839, m (raw "\\sharp"))
    , (chr 9837, m (raw "\\flat"))
    , (chr 9794, raw "\\male")
    , (chr 9792, raw "\\female")
    , (chr 9742, raw "\\phone")
    , (chr 9742, raw "\\ding{37}")
    , (chr 8981, raw "\\recorder")
    , (chr 8471, raw "\\textcircledP")
    , (chr 8471, raw "\\textcircled{P}")
    , (chr 8471, raw "\\textcircled{\\scshape p}")
    , (chr 8257, m (raw "\\mathchar\"1356"))
    , (chr 8218, raw ",")
    , (chr 8218, raw "\\quotesinglbase")
    , (chr 8222, raw ",,")
    , (chr 8222, raw "\\quotedblbase")
    , (chr 64256, raw "ff")
    , (chr 64257, raw "fi")
    , (chr 58290, raw "fj")
    , (chr 64259, raw "ffi")
    , (chr 64260, raw "ffl")
    , (chr 64258, raw "fl")
    , (chr 8230, raw "\\mldr")
    , (chr 8223, raw "``")
    , (chr 8223, raw "\\textquotedblleft")
    , (chr 8219, raw "`")
    , (chr 8942, m (raw "\\vdots"))
    , (chr 8259, raw "\\hybull")
    , (chr 9674, m (raw "\\lozenge"))
    , (chr 59403, m (raw "\\blacklozenge"))
    , (chr 9667, m (raw "\\triangleleft"))
    , (chr 9657, m (raw "\\triangleright"))
    , (chr 9733, m (raw "\\bigstar"))
    , (chr 9838, m (raw "\\natural"))
    , (chr 8478, raw "\\textrecipe")
    , (chr 10038, raw "\\ding{86}")
    , (chr 8982, m (raw "\\mathchar\"2208"))
    , (chr 8973, raw "\\dlcrop")
    , (chr 8972, raw "\\drcrop")
    , (chr 8975, raw "\\ulcrop")
    , (chr 8974, raw "\\urcrop")
    , (chr 189, m (raw "\\sfrac{1}{2}"))
    , (chr 189, m (raw "\\frac{1}{2}"))
    , (chr 188, m (raw "\\sfrac{1}{4}"))
    , (chr 188, m (raw "\\frac{1}{4}"))
    , (chr 190, m (raw "\\sfrac{3}{4}"))
    , (chr 190, m (raw "\\frac{3}{4}"))
    , (chr 8539, m (raw "\\sfrac{1}{8}"))
    , (chr 8539, m (raw "\\frac{1}{8}"))
    , (chr 8540, m (raw "\\sfrac{3}{8}"))
    , (chr 8540, m (raw "\\frac{3}{8}"))
    , (chr 8541, m (raw "\\sfrac{5}{8}"))
    , (chr 8541, m (raw "\\frac{5}{8}"))
    , (chr 8542, m (raw "\\sfrac{7}{8}"))
    , (chr 8542, m (raw "\\frac{7}{8}"))
    , (chr 185, m (raw "^1"))
    , (chr 178, m (raw "^2"))
    , (chr 179, m (raw "^3"))
    , (chr 43, m (raw "+"))
    , (chr 177, raw "\\textpm")
    , (chr 177, m (raw "\\pm"))
    , (chr 60, raw "\\textless")
    , (chr 60, m (raw "<"))
    , (chr 61, m (raw "="))
    , (chr 62, raw "\\textgreater")
    , (chr 62, m (raw ">"))
    , (chr 247, raw "\\textdiv")
    , (chr 247, m (raw "\\div"))
    , (chr 215, raw "\\texttimes")
    , (chr 215, m (raw "\\times"))
    , (chr 164, raw "\\textcurrency")
    , (chr 163, raw "\\pounds")
    , (chr 163, raw "\\textsterling")
    , (chr 163, m (raw "\\mathsterling"))
    , (chr 36, raw "\\$")
    , (chr 36, raw "\\textdollar")
    , (chr 162, raw "\\textcent")
    , (chr 165, raw "\\textyen")
    , (chr 35, raw "\\#")
    , (chr 37, raw "\\%")
    , (chr 38, raw "\\&")
    , (chr 42, m (raw "\\ast"))
    , (chr 64, raw "@")
    , (chr 91, raw "[")
    , (chr 92, raw "\\textbackslash")
    , (chr 92, m (raw "\\backslash"))
    , (chr 93, raw "]")
    , (chr 123, raw "\\{")
    , (chr 123, raw "\\textbraceleft")
    , (chr 123, m (raw "\\lbrace"))
    , (chr 8213, raw "---")
    , (chr 8213, raw "\\rule{1em}{1pt}")
    , (chr 124, raw "\\textbar")
    , (chr 124, m (raw "\\vert"))
    , (chr 124, m (raw "|"))
    , (chr 125, raw "\\}")
    , (chr 125, raw "$\\rbrace$")
    , (chr 181, raw "\\textmu")
    , (chr 181, m (raw "\\mathrm{\\mu}"))
    , (chr 8486, raw "\\textohm")
    , (chr 8486, m (raw "\\Omega"))
    , (chr 176, raw "\\textdegree")
    , (chr 176, raw "\\textdegree")
    , (chr 176, m (raw "^\\circ"))
    , (chr 186, raw "\\textordmasculine")
    , (chr 186, raw "\\textordmasculine")
    , (chr 170, raw "\\textordfeminine")
    , (chr 170, raw "\\textordfeminine")
    , (chr 167, raw "\\S")
    , (chr 167, raw "\\textsection")
    , (chr 182, raw "\\P")
    , (chr 182, raw "\\textparagraph")
    , (chr 182, raw "\\textpilcrow")
    , (chr 183, raw "\\textperiodcentered")
    , (chr 183, m (raw "\\cdot"))
    , (chr 183, m (raw "\\cdotp"))
    , (chr 8592, raw "\\textleftarrow")
    , (chr 8592, m (raw "\\leftarrow"))
    , (chr 8594, raw "\\textrightarrow")
    , (chr 8594, m (raw "\\rightarrow"))
    , (chr 8593, raw "\\textuparrow")
    , (chr 8593, m (raw "\\uparrow"))
    , (chr 8595, raw "\\textdownarrow")
    , (chr 8595, m (raw "\\downarrow"))
    , (chr 169, raw "\\copyright")
    , (chr 169, raw "\\textcopyright")
    , (chr 174, raw "\\textregistered")
    , (chr 8482, raw "\\texttrademark")
    , (chr 166, raw "\\textbrokenbar")
    , (chr 166, raw "\\brokenvert")
    , (chr 172, raw "\\textlnot")
    , (chr 172, m (raw "\\lnot"))
    , (chr 9834, raw "\\textmusicalnote")
    , (chr 9834, raw "\\eighthnote")
    , (chr 33, raw "!")
    , (chr 161, raw "!`")
    , (chr 161, raw "\\textexclamdown")
    , (chr 34, raw "\"")
    , (chr 34, raw "\\textquotedbl")
    , (chr 39, raw "'")
    , (chr 39, raw "\\textquotesingle")
    , (chr 40, raw "(")
    , (chr 41, raw ")")
    , (chr 44, raw ",")
    , (chr 95, raw "\\underline{\\hspace{1ex}}")
    , (chr 95, raw "\\_")
    , (chr 95, raw "\\textunderscore")
    , (chr 45, raw "-")
    , (chr 46, raw ".")
    , (chr 46, raw "\\@.")
    , (chr 46, m (raw "\\ldotp"))
    , (chr 47, raw "/")
    , (chr 58, raw ":")
    , (chr 58, m (raw "\\colon"))
    , (chr 59, raw ";")
    , (chr 63, raw "?")
    , (chr 191, raw "?`")
    , (chr 191, raw "\\textquestiondown")
    , (chr 171, raw "\\guillemotleft")
    , (chr 187, raw "\\guillemotright")
    , (chr 8216, raw "`")
    , (chr 8216, raw "\\textquoteleft")
    , (chr 8217, raw "'")
    , (chr 8217, raw "\\textquoteright")
    , (chr 8220, raw "``")
    , (chr 8220, raw "\\textquotedblleft")
    , (chr 8221, raw "''")
    , (chr 8221, raw "\\textquotedblright")
    , (chr 160, raw "~")
    , (chr 173, raw "\\-")
    , (chr 9472, raw "\\boxh")
    , (chr 9474, raw "\\boxv")
    , (chr 9492, raw "\\boxur")
    , (chr 9496, raw "\\boxul")
    , (chr 9488, raw "\\boxdl")
    , (chr 9484, raw "\\boxdr")
    , (chr 9500, raw "\\boxvr")
    , (chr 9524, raw "\\boxhu")
    , (chr 9508, raw "\\boxvl")
    , (chr 9516, raw "\\boxhd")
    , (chr 9532, raw "\\boxvh")
    , (chr 9566, raw "\\boxvR")
    , (chr 9576, raw "\\boxhU")
    , (chr 9569, raw "\\boxvL")
    , (chr 9573, raw "\\boxhD")
    , (chr 9578, raw "\\boxvH")
    , (chr 9552, raw "\\boxH")
    , (chr 9553, raw "\\boxV")
    , (chr 9562, raw "\\boxUR")
    , (chr 9565, raw "\\boxUL")
    , (chr 9559, raw "\\boxDL")
    , (chr 9556, raw "\\boxDR")
    , (chr 9568, raw "\\boxVR")
    , (chr 9577, raw "\\boxHU")
    , (chr 9571, raw "\\boxVL")
    , (chr 9574, raw "\\boxHD")
    , (chr 9580, raw "\\boxVH")
    , (chr 9567, raw "\\boxVr")
    , (chr 9575, raw "\\boxHu")
    , (chr 9570, raw "\\boxVl")
    , (chr 9572, raw "\\boxHd")
    , (chr 9579, raw "\\boxVh")
    , (chr 9560, raw "\\boxuR")
    , (chr 9564, raw "\\boxUl")
    , (chr 9557, raw "\\boxdL")
    , (chr 9555, raw "\\boxDr")
    , (chr 9561, raw "\\boxUr")
    , (chr 9563, raw "\\boxuL")
    , (chr 9558, raw "\\boxDl")
    , (chr 9554, raw "\\boxdR")
    , (chr 8501, m (raw "\\aleph"))
    , (chr 8743, m (raw "\\wedge"))
    , (chr 8743, m (raw "\\land"))
    , (chr 8738, m (raw "\\sphericalangle"))
    , (chr 8776, m (raw "\\approx"))
    , (chr 8757, m (raw "\\because"))
    , (chr 8869, m (raw "\\bot"))
    , (chr 8745, m (raw "\\cap"))
    , (chr 8773, m (raw "\\cong"))
    , (chr 8750, m (raw "\\oint"))
    , (chr 8746, m (raw "\\cup"))
    , (chr 8801, m (raw "\\equiv"))
    , (chr 8707, m (raw "\\exists"))
    , (chr 8704, m (raw "\\forall"))
    , (chr 402, m (raw "f"))
    , (chr 8805, m (raw "\\geq"))
    , (chr 8805, m (raw "\\ge"))
    , (chr 8660, m (raw "\\iff"))
    , (chr 8734, m (raw "\\infty"))
    , (chr 8747, m (raw "\\int"))
    , (chr 8712, m (raw "\\in"))
    , (chr 12296, raw "\\textlangle")
    , (chr 12296, m (raw "\\langle"))
    , (chr 8656, m (raw "\\Leftarrow"))
    , (chr 8804, m (raw "\\leq"))
    , (chr 8804, m (raw "\\le"))
    , (chr 8722, raw "\\textminus")
    , (chr 8722, m (raw "-"))
    , (chr 8723, m (raw "\\mp"))
    , (chr 8711, m (raw "\\nabla"))
    , (chr 8800, m (raw "\\not="))
    , (chr 8800, m (raw "\\neq"))
    , (chr 8800, m (raw "\\ne"))
    , (chr 8715, m (raw "\\ni"))
    , (chr 8715, m (raw "\\owns"))
    , (chr 8744, m (raw "\\vee"))
    , (chr 8744, m (raw "\\lor"))
    , (chr 8741, m (raw "\\parallel"))
    , (chr 8706, m (raw "\\partial"))
    , (chr 8240, raw "\\textperthousand")
    , (chr 8869, m (raw "\\perp"))
    , (chr 8242, m (raw "^\\prime"))
    , (chr 8243, m (raw "{''}"))
    , (chr 8733, m (raw "\\propto"))
    , (chr 8730, raw "\\textsurd")
    , (chr 8730, m (raw "\\surd"))
    , (chr 12297, raw "\\textrangle")
    , (chr 12297, m (raw "\\rangle"))
    , (chr 8658, m (raw "\\Rightarrow"))
    , (chr 8764, m (raw "\\sim"))
    , (chr 8771, m (raw "\\simeq"))
    , (chr 9632, m (raw "\\square"))
    , (chr 8834, m (raw "\\subset"))
    , (chr 8838, m (raw "\\subseteq"))
    , (chr 8835, m (raw "\\supset"))
    , (chr 8839, m (raw "\\supseteq"))
    , (chr 8756, m (raw "\\therefore"))
    , (chr 8214, raw "\\textbardbl")
    , (chr 8214, m (raw "\\Vert"))
    , (chr 8214, m (raw "\\|"))
    , (chr 8491, raw "\\AA")
    , (chr 8492, m (raw "B"))
    , (chr 8492, m (raw "\\mathcal{B}"))
    , (chr 8728, m (raw "\\circ"))
    , (chr 168, m (raw "\\ddot{}"))
    , (chr 8412, m (raw "\\ddot{}\\kern4.5pt\\ddot{ }"))
    , (chr 8459, m (raw "H"))
    , (chr 8459, m (raw "\\mathcal{H}"))
    , (chr 8466, m (raw "L"))
    , (chr 8466, m (raw "\\mathcal{L}"))
    , (chr 8727, m (raw "_\\ast"))
    , (chr 8713, m (raw "\\not\\in"))
    , (chr 8500, m (raw "\\mathit{o}"))
    , (chr 8499, m (raw "M"))
    , (chr 8499, m (raw "\\mathcal{M}"))
    , (chr 8411, m (raw "\\ddot{}\\kern 3pt\\dot{ }"))
    , (chr 8244, m (raw "{'''}"))
    , (chr 8793, m (raw "\\stackrel{\\wedge}{=}"))
    , (chr 945, m (raw "\\alpha"))
    , (chr 946, m (raw "\\beta"))
    , (chr 947, m (raw "\\gamma"))
    , (chr 915, m (raw "\\Gamma"))
    , (chr 948, m (raw "\\delta"))
    , (chr 916, m (raw "\\Delta"))
    , (chr 949, m (raw "\\epsilon"))
    , (chr 8714, m (raw "\\epsilon"))
    , (chr 950, m (raw "\\zeta"))
    , (chr 951, m (raw "\\eta"))
    , (chr 952, m (raw "\\theta"))
    , (chr 920, m (raw "\\Theta"))
    , (chr 977, m (raw "\\vartheta"))
    , (chr 953, m (raw "\\iota"))
    , (chr 954, m (raw "\\kappa"))
    , (chr 955, m (raw "\\lambda"))
    , (chr 923, m (raw "\\Lambda"))
    , (chr 956, m (raw "\\mu"))
    , (chr 957, m (raw "\\nu"))
    , (chr 958, m (raw "\\xi"))
    , (chr 926, m (raw "\\Xi"))
    , (chr 960, m (raw "\\pi"))
    , (chr 982, m (raw "\\varpi"))
    , (chr 928, m (raw "\\Pi"))
    , (chr 961, m (raw "\\rho"))
    , (chr 1009, m (raw "\\varrho"))
    , (chr 963, m (raw "\\sigma"))
    , (chr 931, m (raw "\\Sigma"))
    , (chr 962, m (raw "\\varsigma"))
    , (chr 964, m (raw "\\tau"))
    , (chr 965, m (raw "\\upsilon"))
    , (chr 978, m (raw "\\Upsilon"))
    , (chr 966, m (raw "\\phi"))
    , (chr 934, m (raw "\\Phi"))
    , (chr 981, m (raw "\\varphi"))
    , (chr 967, m (raw "\\chi"))
    , (chr 968, m (raw "\\psi"))
    , (chr 936, m (raw "\\Psi"))
    , (chr 969, m (raw "\\omega"))
    , (chr 937, m (raw "\\Omega"))
    , (chr 945, m (raw "\\alpha"))
    , (chr 946, m (raw "\\beta"))
    , (chr 947, m (raw "\\gamma"))
    , (chr 915, m (raw "\\Gamma"))
    , (chr 948, m (raw "\\delta"))
    , (chr 916, m (raw "\\Delta"))
    , (chr 949, m (raw "\\epsilon"))
    , (chr 949, m (raw "\\varepsilon"))
    , (chr 949, m (raw "\\epsilon"))
    , (chr 950, m (raw "\\zeta"))
    , (chr 951, m (raw "\\eta"))
    , (chr 952, m (raw "\\theta"))
    , (chr 920, m (raw "\\Theta"))
    , (chr 977, m (raw "\\vartheta"))
    , (chr 953, m (raw "\\iota"))
    , (chr 954, m (raw "\\kappa"))
    , (chr 955, m (raw "\\lambda"))
    , (chr 923, m (raw "\\Lambda"))
    , (chr 956, m (raw "\\mu"))
    , (chr 957, m (raw "\\nu"))
    , (chr 958, m (raw "\\xi"))
    , (chr 926, m (raw "\\Xi"))
    , (chr 960, m (raw "\\pi"))
    , (chr 928, m (raw "\\Pi"))
    , (chr 982, m (raw "\\varpi"))
    , (chr 961, m (raw "\\rho"))
    , (chr 1009, m (raw "\\varrho"))
    , (chr 963, m (raw "\\sigma"))
    , (chr 931, m (raw "\\Sigma"))
    , (chr 962, m (raw "\\varsigma"))
    , (chr 964, m (raw "\\tau"))
    , (chr 965, m (raw "\\upsilon"))
    , (chr 978, m (raw "\\Upsilon"))
    , (chr 966, m (raw "\\phi"))
    , (chr 934, m (raw "\\Phi"))
    , (chr 981, m (raw "\\varphi"))
    , (chr 967, m (raw "\\chi"))
    , (chr 968, m (raw "\\psi"))
    , (chr 936, m (raw "\\Psi"))
    , (chr 969, m (raw "\\omega"))
    , (chr 937, m (raw "\\Omega"))
    , (chr 8736, m (raw "\\angle"))
    , (chr 8737, m (raw "\\measuredangle"))
    , (chr 8705, m (raw "\\complement"))
    , (chr 8467, m (raw "\\ell"))
    , (chr 8709, m (raw "\\emptyset"))
    , (chr 8465, m (raw "\\Im"))
    , (chr 8708, m (raw "\\nexists"))
    , (chr 9416, raw "\\textcircled{S}")
    , (chr 9416, m (raw "\\circledS"))
    , (chr 8463, m (raw "\\hbar"))
    , (chr 8476, m (raw "\\Re"))
    , (chr 65128, m (raw "\\smallsetminus"))
    , (chr 8472, m (raw "\\wp"))
    , (chr 8720, m (raw "\\amalg"))
    , (chr 8966, m (raw "\\doublebarwedge"))
    , (chr 8892, m (raw "\\barwedge"))
    , (chr 8914, m (raw "\\Cap"))
    , (chr 8914, m (raw "\\doublecap"))
    , (chr 8915, m (raw "\\Cup"))
    , (chr 8915, m (raw "\\doublecup"))
    , (chr 8910, m (raw "\\curlyvee"))
    , (chr 8911, m (raw "\\curlywedge"))
    , (chr 8900, m (raw "\\diamond"))
    , (chr 8903, m (raw "\\divideontimes"))
    , (chr 8890, m (raw "\\intercal"))
    , (chr 8907, m (raw "\\leftthreetimes"))
    , (chr 8905, m (raw "\\ltimes"))
    , (chr 8863, m (raw "\\boxminus"))
    , (chr 8859, m (raw "\\circledast"))
    , (chr 8858, m (raw "\\circledcirc"))
    , (chr 8861, m (raw "\\circleddash"))
    , (chr 8857, m (raw "\\odot"))
    , (chr 8854, m (raw "\\ominus"))
    , (chr 8853, m (raw "\\oplus"))
    , (chr 8856, m (raw "\\oslash"))
    , (chr 8855, m (raw "\\otimes"))
    , (chr 8862, m (raw "\\boxplus"))
    , (chr 8724, m (raw "\\dotplus"))
    , (chr 8908, m (raw "\\rightthreetimes"))
    , (chr 8906, m (raw "\\rtimes"))
    , (chr 8901, m (raw "\\cdot"))
    , (chr 8865, m (raw "\\boxdot"))
    , (chr 8726, m (raw "\\setminus"))
    , (chr 8851, m (raw "\\sqcap"))
    , (chr 8852, m (raw "\\sqcup"))
    , (chr 8902, m (raw "\\star"))
    , (chr 8864, m (raw "\\boxtimes"))
    , (chr 8868, m (raw "\\top"))
    , (chr 8846, m (raw "\\uplus"))
    , (chr 8768, m (raw "\\wr"))
    , (chr 9711, raw "\\textbigcircle")
    , (chr 9711, m (raw "\\bigcirc"))
    , (chr 9661, m (raw "\\bigtriangledown"))
    , (chr 9651, m (raw "\\bigtriangleup"))
    , (chr 8720, m (raw "\\coprod"))
    , (chr 8719, m (raw "\\prod"))
    , (chr 8721, m (raw "\\sum"))
    , (chr 8778, m (raw "\\approxeq"))
    , (chr 8781, m (raw "\\asymp"))
    , (chr 8780, m (raw "\\backcong"))
    , (chr 8717, m (raw "\\backepsilon"))
    , (chr 8904, m (raw "\\bowtie"))
    , (chr 8765, m (raw "\\backsim"))
    , (chr 8909, m (raw "\\backsimeq"))
    , (chr 8782, m (raw "\\Bumpeq"))
    , (chr 8783, m (raw "\\bumpeq"))
    , (chr 8791, m (raw "\\circeq"))
    , (chr 8788, m (raw ":="))
    , (chr 8926, m (raw "\\curlyeqprec"))
    , (chr 8927, m (raw "\\curlyeqsucc"))
    , (chr 8828, m (raw "\\preccurlyeq"))
    , (chr 8867, m (raw "\\dashv"))
    , (chr 8790, m (raw "\\eqcirc"))
    , (chr 8789, m (raw "=:"))
    , (chr 8785, m (raw "\\doteqdot"))
    , (chr 8785, m (raw "\\Doteq"))
    , (chr 8784, m (raw "\\doteq"))
    , (chr 8786, m (raw "\\fallingdotseq"))
    , (chr 8925, m (raw "\\eqslantgtr"))
    , (chr 8924, m (raw "\\eqslantless"))
    , (chr 8787, m (raw "\\risingdotseq"))
    , (chr 8916, m (raw "\\pitchfork"))
    , (chr 8994, m (raw "\\frown"))
    , (chr 8919, m (raw "\\gtrdot"))
    , (chr 8807, m (raw "\\geqq"))
    , (chr 8923, m (raw "\\gtreqless"))
    , (chr 8923, m (raw "\\gtreqqless"))
    , (chr 8921, m (raw "\\ggg"))
    , (chr 8921, m (raw "\\gggtr"))
    , (chr 8823, m (raw "\\gtrless"))
    , (chr 8819, m (raw "\\gtrsim"))
    , (chr 8811, m (raw "\\gg"))
    , (chr 8918, m (raw "\\lessdot"))
    , (chr 8806, m (raw "\\leqq"))
    , (chr 8922, m (raw "\\lesseqqgtr"))
    , (chr 8922, m (raw "\\lesseqgtr"))
    , (chr 8822, m (raw "\\lessgtr"))
    , (chr 8920, m (raw "\\lll"))
    , (chr 8920, m (raw "\\llless"))
    , (chr 8818, m (raw "\\lesssim"))
    , (chr 8810, m (raw "\\ll"))
    , (chr 8884, m (raw "\\trianglelefteq"))
    , (chr 8739, m (raw "\\mid"))
    , (chr 8871, m (raw "\\models"))
    , (chr 8826, m (raw "\\prec"))
    , (chr 8828, m (raw "\\preceq"))
    , (chr 8830, m (raw "\\precsim"))
    , (chr 8885, m (raw "\\trianglerighteq"))
    , (chr 8827, m (raw "\\succ"))
    , (chr 8829, m (raw "\\succcurlyeq"))
    , (chr 8829, m (raw "\\succeq"))
    , (chr 8831, m (raw "\\succsim"))
    , (chr 8995, m (raw "\\smile"))
    , (chr 8847, m (raw "\\sqsubset"))
    , (chr 8849, m (raw "\\sqsubseteq"))
    , (chr 8848, m (raw "\\sqsupset"))
    , (chr 8850, m (raw "\\sqsupseteq"))
    , (chr 8912, m (raw "\\Subset"))
    , (chr 8838, m (raw "\\subseteqq"))
    , (chr 8913, m (raw "\\Supset"))
    , (chr 8839, m (raw "\\supseteqq"))
    , (chr 8796, m (raw "\\triangleq"))
    , (chr 8812, m (raw "\\between"))
    , (chr 8866, m (raw "\\vdash"))
    , (chr 8873, m (raw "\\Vdash"))
    , (chr 8872, m (raw "\\vDash"))
    , (chr 8891, m (raw "\\veebar"))
    , (chr 8882, m (raw "\\vartriangleleft"))
    , (chr 8733, m (raw "\\varpropto"))
    , (chr 8883, m (raw "\\vartriangleright"))
    , (chr 8874, m (raw "\\Vvdash"))
    , (chr 8809, m (raw "\\gneq"))
    , (chr 8809, m (raw "\\gneqq"))
    , (chr 8935, m (raw "\\gnsim"))
    , (chr 8809, m (raw "\\gvertneqq"))
    , (chr 8808, m (raw "\\lneqq"))
    , (chr 8808, m (raw "\\lneq"))
    , (chr 8934, m (raw "\\lnsim"))
    , (chr 8808, m (raw "\\lvertneqq"))
    , (chr 8777, m (raw "\\not\\approx"))
    , (chr 8775, m (raw "\\not\\cong"))
    , (chr 8775, m (raw "\\ncong"))
    , (chr 8802, m (raw "\\not\\equiv"))
    , (chr 8817, m (raw "\\not\\geq"))
    , (chr 8817, m (raw "\\ngeq"))
    , (chr 8817, m (raw "\\ngeqslant"))
    , (chr 8815, m (raw "\\not>"))
    , (chr 8815, m (raw "\\ngtr"))
    , (chr 8816, m (raw "\\not\\leq"))
    , (chr 8816, m (raw "\\nleq"))
    , (chr 8816, m (raw "\\nleqslant"))
    , (chr 8814, m (raw "\\not<"))
    , (chr 8814, m (raw "\\nless"))
    , (chr 8938, m (raw "\\ntriangleleft"))
    , (chr 8940, m (raw "\\ntrianglelefteq"))
    , (chr 8740, m (raw "\\nmid"))
    , (chr 8742, m (raw "\\nparallel"))
    , (chr 8832, m (raw "\\not\\prec"))
    , (chr 8832, m (raw "\\nprec"))
    , (chr 8928, m (raw "\\not\\preceq"))
    , (chr 8928, m (raw "\\npreceq"))
    , (chr 8939, m (raw "\\ntriangleright"))
    , (chr 8941, m (raw "\\ntrianglerighteq"))
    , (chr 8833, m (raw "\\not\\succ"))
    , (chr 8833, m (raw "\\nsucc"))
    , (chr 8929, m (raw "\\not\\succeq"))
    , (chr 8929, m (raw "\\nsucceq"))
    , (chr 8769, m (raw "\\not\\sim"))
    , (chr 8769, m (raw "\\nsim"))
    , (chr 8772, m (raw "\\not\\simeq"))
    , (chr 8836, m (raw "\\not\\subset"))
    , (chr 8840, m (raw "\\not\\subseteq"))
    , (chr 8840, m (raw "\\nsubseteq"))
    , (chr 8840, m (raw "\\nsubseteqq"))
    , (chr 8837, m (raw "\\not\\supset"))
    , (chr 8841, m (raw "\\nsupseteqq"))
    , (chr 8841, m (raw "\\not\\supseteq"))
    , (chr 8841, m (raw "\\nsupseteq"))
    , (chr 8876, m (raw "\\nvdash"))
    , (chr 8877, m (raw "\\nvDash"))
    , (chr 8879, m (raw "\\nVDash"))
    , (chr 8878, m (raw "\\nVdash"))
    , (chr 8936, m (raw "\\precnsim"))
    , (chr 8937, m (raw "\\succnsim"))
    , (chr 8842, m (raw "\\subsetneq"))
    , (chr 8842, m (raw "\\subsetneqq"))
    , (chr 8843, m (raw "\\supsetneq"))
    , (chr 8843, m (raw "\\supsetneqq"))
    , (chr 8630, m (raw "\\curvearrowleft"))
    , (chr 8631, m (raw "\\curvearrowright"))
    , (chr 8659, m (raw "\\Downarrow"))
    , (chr 8650, m (raw "\\downdownarrows"))
    , (chr 8643, m (raw "\\downharpoonleft"))
    , (chr 8642, m (raw "\\downharpoonright"))
    , (chr 8666, m (raw "\\Lleftarrow"))
    , (chr 8606, m (raw "\\twoheadleftarrow"))
    , (chr 8647, m (raw "\\leftleftarrows"))
    , (chr 8617, m (raw "\\hookleftarrow"))
    , (chr 8619, m (raw "\\looparrowleft"))
    , (chr 8610, m (raw "\\leftarrowtail"))
    , (chr 8637, m (raw "\\leftharpoondown"))
    , (chr 8636, m (raw "\\leftharpoonup"))
    , (chr 8660, m (raw "\\Leftrightarrow"))
    , (chr 8596, m (raw "\\leftrightarrow"))
    , (chr 8646, m (raw "\\leftrightarrows"))
    , (chr 8644, m (raw "\\rightleftarrows"))
    , (chr 8621, m (raw "\\leftrightsquigarrow"))
    , (chr 8652, m (raw "\\rightleftharpoons"))
    , (chr 8651, m (raw "\\leftrightharpoons"))
    , (chr 8624, m (raw "\\Lsh"))
    , (chr 8614, m (raw "\\mapsto"))
    , (chr 8888, m (raw "\\multimap"))
    , (chr 8599, m (raw "\\nearrow"))
    , (chr 8653, m (raw "\\nLeftarrow"))
    , (chr 8654, m (raw "\\nLeftrightarrow"))
    , (chr 8622, m (raw "\\nleftrightarrow"))
    , (chr 8655, m (raw "\\nRightarrow"))
    , (chr 8598, m (raw "\\nwarrow"))
    , (chr 8634, m (raw "\\circlearrowleft"))
    , (chr 8635, m (raw "\\circlearrowright"))
    , (chr 8667, m (raw "\\Rrightarrow"))
    , (chr 8608, m (raw "\\twoheadrightarrow"))
    , (chr 8649, m (raw "\\rightrightarrows"))
    , (chr 8618, m (raw "\\hookrightarrow"))
    , (chr 8620, m (raw "\\looparrowright"))
    , (chr 8611, m (raw "\\rightarrowtail"))
    , (chr 8669, m (raw "\\rightsquigarrow"))
    , (chr 8641, m (raw "\\rightharpoondown"))
    , (chr 8640, m (raw "\\rightharpoonup"))
    , (chr 8625, m (raw "\\Rsh"))
    , (chr 8600, m (raw "\\searrow"))
    , (chr 8601, m (raw "\\swarrow"))
    , (chr 8657, m (raw "\\Uparrow"))
    , (chr 8648, m (raw "\\upuparrows"))
    , (chr 8661, m (raw "\\Updownarrow"))
    , (chr 8597, m (raw "\\updownarrow"))
    , (chr 8639, m (raw "\\upharpoonleft"))
    , (chr 8638, m (raw "\\upharpoonright"))
    , (chr 8969, m (raw "\\rceil"))
    , (chr 8971, m (raw "\\rfloor"))
    , (chr 8989, m (raw "\\urcorner"))
    , (chr 8991, m (raw "\\lrcorner"))
    , (chr 8968, m (raw "\\lceil"))
    , (chr 8970, m (raw "\\lfloor"))
    , (chr 8988, m (raw "\\ulcorner"))
    , (chr 8990, m (raw "\\llcorner"))
    , (chr 225, raw "\\'{a}")
    , (chr 193, raw "\\'{A}")
    , (chr 226, raw "\\^{a}")
    , (chr 194, raw "\\^{A}")
    , (chr 224, raw "\\`{a}")
    , (chr 192, raw "\\`{A}")
    , (chr 229, raw "\\aa")
    , (chr 197, raw "\\AA")
    , (chr 227, raw "\\~{a}")
    , (chr 195, raw "\\~{A}")
    , (chr 228, raw "\\\"{a}")
    , (chr 196, raw "\\\"{A}")
    , (chr 230, raw "\\ae")
    , (chr 198, raw "\\AE")
    , (chr 231, raw "\\c{c}")
    , (chr 199, raw "\\c{C}")
    , (chr 240, raw "\\dh")
    , (chr 208, raw "\\DH")
    , (chr 208, raw "\\DH")
    , (chr 233, raw "\\'{e}")
    , (chr 201, raw "\\'{E}")
    , (chr 234, raw "\\^{e}")
    , (chr 202, raw "\\^{E}")
    , (chr 232, raw "\\`{e}")
    , (chr 200, raw "\\`{E}")
    , (chr 235, raw "\\\"{e}")
    , (chr 203, raw "\\\"{E}")
    , (chr 237, raw "\\'{\\i}")
    , (chr 205, raw "\\'{I}")
    , (chr 238, raw "\\^{\\i}")
    , (chr 206, raw "\\^{I}")
    , (chr 236, raw "\\`{\\i}")
    , (chr 204, raw "\\`{I}")
    , (chr 239, raw "\\\"{\\i}")
    , (chr 207, raw "\\\"{I}")
    , (chr 241, raw "\\~{n}")
    , (chr 209, raw "\\~{N}")
    , (chr 243, raw "\\'{o}")
    , (chr 211, raw "\\'{O}")
    , (chr 244, raw "\\^{o}")
    , (chr 212, raw "\\^{O}")
    , (chr 242, raw "\\`{o}")
    , (chr 210, raw "\\`{O}")
    , (chr 248, raw "\\o")
    , (chr 216, raw "\\O")
    , (chr 245, raw "\\~{o}")
    , (chr 213, raw "\\~{O}")
    , (chr 246, raw "\\\"{o}")
    , (chr 214, raw "\\\"{O}")
    , (chr 223, raw "\\ss")
    , (chr 254, raw "\\th")
    , (chr 222, raw "\\TH")
    , (chr 250, raw "\\'{u}")
    , (chr 218, raw "\\'{U}")
    , (chr 251, raw "\\^{u}")
    , (chr 219, raw "\\^{U}")
    , (chr 249, raw "\\`{u}")
    , (chr 217, raw "\\`{U}")
    , (chr 252, raw "\\\"{u}")
    , (chr 220, raw "\\\"{U}")
    , (chr 253, raw "\\'{y}")
    , (chr 221, raw "\\'{Y}")
    , (chr 255, raw "\\\"{y}")
    , (chr 259, raw "\\u{a}")
    , (chr 258, raw "\\u{A}")
    , (chr 257, raw "\\={a}")
    , (chr 256, raw "\\={A}")
    , (chr 261, raw "\\k{a}")
    , (chr 260, raw "\\k{A}")
    , (chr 263, raw "\\'{c}")
    , (chr 262, raw "\\'{C}")
    , (chr 269, raw "\\v{c}")
    , (chr 268, raw "\\v{C}")
    , (chr 265, raw "\\^{c}")
    , (chr 264, raw "\\^{C}")
    , (chr 267, raw "\\.{c}")
    , (chr 266, raw "\\.{C}")
    , (chr 271, raw "\\v{d}")
    , (chr 270, raw "\\v{D}")
    , (chr 273, raw "\\dj")
    , (chr 272, raw "\\DJ")
    , (chr 283, raw "\\v{e}")
    , (chr 282, raw "\\v{E}")
    , (chr 279, raw "\\.{e}")
    , (chr 278, raw "\\.{E}")
    , (chr 275, raw "\\={e}")
    , (chr 274, raw "\\={E}")
    , (chr 281, raw "\\k{e}")
    , (chr 280, raw "\\k{E}")
    , (chr 501, raw "\\'{g}")
    , (chr 287, raw "\\u{g}")
    , (chr 286, raw "\\u{G}")
    , (chr 290, raw "\\c{G}")
    , (chr 285, raw "\\^{g}")
    , (chr 284, raw "\\^{G}")
    , (chr 289, raw "\\.{g}")
    , (chr 288, raw "\\.{G}")
    , (chr 293, raw "\\^{h}")
    , (chr 292, raw "\\^{H}")
    , (chr 295, raw "\\hstrok")
    , (chr 294, raw "\\Hstrok")
    , (chr 304, raw "\\.{I}")
    , (chr 298, raw "\\={I}")
    , (chr 299, raw "\\={\\i}")
    , (chr 307, raw "i\\kern -.15em j")
    , (chr 306, raw "I\\kern -.15em J")
    , (chr 305, raw "\\i")
    , (chr 303, raw "\\k{i}")
    , (chr 302, raw "\\k{I}")
    , (chr 297, raw "\\~{\\i}")
    , (chr 296, raw "\\~{I}")
    , (chr 309, raw "\\^{\\j}")
    , (chr 308, raw "\\^{J}")
    , (chr 311, raw "\\c{k}")
    , (chr 310, raw "\\c{K}")
    , (chr 312, raw "\\textsc{k}")
    , (chr 314, raw "\\'{l}")
    , (chr 313, raw "\\'{L}")
    , (chr 318, raw "\\v{l}")
    , (chr 317, raw "\\v{L}")
    , (chr 316, raw "\\c{l}")
    , (chr 315, raw "\\c{L}")
    , (chr 320, raw "\\lmidot")
    , (chr 319, raw "\\Lmidot")
    , (chr 322, raw "\\l")
    , (chr 321, raw "\\L")
    , (chr 324, raw "\\'{n}")
    , (chr 323, raw "\\'{N}")
    , (chr 331, raw "\\ng")
    , (chr 330, raw "\\NG")
    , (chr 329, raw "n\\kern-.2em\\textsf{'}")
    , (chr 329, raw "n{'}")
    , (chr 328, raw "\\v{n}")
    , (chr 327, raw "\\v{N}")
    , (chr 326, raw "\\c{n}")
    , (chr 325, raw "\\c{N}")
    , (chr 337, raw "\\H{o}")
    , (chr 336, raw "\\H{O}")
    , (chr 332, raw "\\={O}")
    , (chr 333, raw "\\={o}")
    , (chr 339, raw "\\oe")
    , (chr 338, raw "\\OE")
    , (chr 341, raw "\\'{r}")
    , (chr 340, raw "\\'{R}")
    , (chr 345, raw "\\v{r}")
    , (chr 344, raw "\\v{R}")
    , (chr 343, raw "\\c{r}")
    , (chr 342, raw "\\c{R}")
    , (chr 347, raw "\\'{s}")
    , (chr 346, raw "\\'{S}")
    , (chr 353, raw "\\v{s}")
    , (chr 352, raw "\\v{S}")
    , (chr 351, raw "\\c{s}")
    , (chr 350, raw "\\c{S}")
    , (chr 349, raw "\\^{s}")
    , (chr 348, raw "\\^{S}")
    , (chr 357, raw "\\v{t}")
    , (chr 356, raw "\\v{T}")
    , (chr 355, raw "\\c{t}")
    , (chr 354, raw "\\c{T}")
    , (chr 359, raw "\\tstrok")
    , (chr 358, raw "\\Tstrok")
    , (chr 365, raw "\\u{u}")
    , (chr 364, raw "\\u{U}")
    , (chr 369, raw "\\H{u}")
    , (chr 368, raw "\\H{U}")
    , (chr 363, raw "\\={u}")
    , (chr 362, raw "\\={U}")
    , (chr 371, raw "\\k{u}")
    , (chr 370, raw "\\k{U}")
    , (chr 367, raw "\\r{u}")
    , (chr 366, raw "\\r{U}")
    , (chr 361, raw "\\~{u}")
    , (chr 360, raw "\\~{U}")
    , (chr 373, raw "\\^{w}")
    , (chr 372, raw "\\^{W}")
    , (chr 375, raw "\\^{y}")
    , (chr 374, raw "\\^{Y}")
    , (chr 376, raw "\\\"{Y}")
    , (chr 378, raw "\\'{z}")
    , (chr 377, raw "\\'{Z}")
    , (chr 382, raw "\\v{z}")
    , (chr 381, raw "\\v{Z}")
    , (chr 380, raw "\\.{z}")
    , (chr 379, raw "\\.{Z}")
    , (chr 180, raw "\\'{}")
    , (chr 180, raw "\\textasciiacute")
    , (chr 180, m (raw "\\acute{}"))
    , (chr 728, raw "\\u{}")
    , (chr 728, raw "\\textasciibreve")
    , (chr 711, raw "\\v{}")
    , (chr 711, raw "\\textasciicaron")
    , (chr 711, m (raw "\\check{}"))
    , (chr 184, raw "\\c{}")
    , (chr 94, raw "\\^{}")
    , (chr 94, raw "\\textasciicircum")
    , (chr 94, m (raw "\\hat{}"))
    , (chr 733, raw "\\H{}")
    , (chr 733, raw "\\textacutedbl")
    , (chr 168, raw "\\\"{}")
    , (chr 168, raw "\\textasciidieresis")
    , (chr 168, m (raw "\\ddot{}"))
    , (chr 729, raw "\\.{}")
    , (chr 729, m (raw "\\dot{}"))
    , (chr 96, raw "`")
    , (chr 96, raw "\\`{}")
    , (chr 96, raw "\\textasciigrave")
    , (chr 175, raw "\\={}")
    , (chr 175, raw "\\textasciimacron")
    , (chr 175, m (raw "\\bar{}"))
    , (chr 731, raw "\\k{}")
    , (chr 730, raw "\\r{}")
    , (chr 126, raw "\\~{}")
    , (chr 126, raw "\\textasciitilde")
    , (chr 126, raw "\\texttildelow")
    , (chr 126, raw "\\raise.5ex\\hbox{\\texttildelow}")
    , (chr 126, m (raw "\\tilde{}"))
    , (chr 1072, raw "a")
    , (chr 1040, raw "A")
    , (chr 1073, raw "b")
    , (chr 1041, raw "B")
    , (chr 1074, raw "v")
    , (chr 1042, raw "V")
    , (chr 1075, raw "g")
    , (chr 1043, raw "G")
    , (chr 1076, raw "d")
    , (chr 1044, raw "D")
    , (chr 1077, raw "e")
    , (chr 1045, raw "E")
    , (chr 1105, raw "\\\"{e}")
    , (chr 1025, raw "\\\"{E}")
    , (chr 1078, raw "zh")
    , (chr 1046, raw "Zh")
    , (chr 1079, raw "z")
    , (chr 1047, raw "Z")
    , (chr 1080, raw "i")
    , (chr 1048, raw "I")
    , (chr 1081, raw "\\u{i}")
    , (chr 1049, raw "\\u{I}")
    , (chr 1082, raw "k")
    , (chr 1050, raw "K")
    , (chr 1083, raw "l")
    , (chr 1051, raw "L")
    , (chr 1084, raw "m")
    , (chr 1052, raw "M")
    , (chr 1085, raw "n")
    , (chr 1053, raw "N")
    , (chr 1086, raw "o")
    , (chr 1054, raw "O")
    , (chr 1087, raw "p")
    , (chr 1055, raw "P")
    , (chr 1088, raw "r")
    , (chr 1056, raw "R")
    , (chr 1089, raw "s")
    , (chr 1057, raw "S")
    , (chr 1090, raw "t")
    , (chr 1058, raw "T")
    , (chr 1091, raw "u")
    , (chr 1059, raw "U")
    , (chr 1092, raw "f")
    , (chr 1060, raw "F")
    , (chr 1093, raw "kh")
    , (chr 1061, raw "Kh")
    , (chr 1094, raw "ts")
    , (chr 1062, raw "Ts")
    , (chr 1095, raw "ch")
    , (chr 1063, raw "Ch")
    , (chr 1096, raw "sh")
    , (chr 1064, raw "Sh")
    , (chr 1097, raw "shch")
    , (chr 1065, raw "Shch")
    , (chr 1099, raw "y")
    , (chr 1067, raw "Y")
    , (chr 1101, raw "\\`{e}")
    , (chr 1069, raw "\\`{E}")
    , (chr 1102, raw "yu")
    , (chr 1070, raw "Yu")
    , (chr 1103, raw "ya")
    , (chr 1071, raw "Ya")
    , (chr 8470, raw "N0")
    , (chr 8470, raw "\\textnumero")
    , (chr 8470, raw "\\textnumero")
    , (chr 1106, raw "dj")
    , (chr 1026, raw "Dj")
    , (chr 1107, raw "\\'g")
    , (chr 1027, raw "\\'G")
    , (chr 1108, raw "\\=e")
    , (chr 1028, raw "\\=E")
    , (chr 1029, raw "\\Dz")
    , (chr 1110, raw "\\=\\i")
    , (chr 1030, raw "\\=I")
    , (chr 1111, raw "\\\"\\i")
    , (chr 1031, raw "\\\"I")
    , (chr 1112, raw "j")
    , (chr 1032, raw "J")
    , (chr 1113, raw "lj")
    , (chr 1033, raw "Lj")
    , (chr 1114, raw "nj")
    , (chr 1034, raw "Nj")
    , (chr 1115, raw "\\'c")
    , (chr 1035, raw "\\'C")
    , (chr 1116, raw "\\'k")
    , (chr 1036, raw "\\'K")
    , (chr 1118, raw "\\u{u}")
    , (chr 1038, raw "\\u{U}")
    , (chr 1119, raw "\\dzh")
    , (chr 1039, raw "\\Dzh")
    , (chr 945, raw "a")
    , (chr 913, raw "A")
    , (chr 913, raw "A")
    , (chr 946, raw "b")
    , (chr 914, raw "B")
    , (chr 914, raw "B")
    , (chr 947, raw "g")
    , (chr 915, raw "G")
    , (chr 948, raw "d")
    , (chr 916, raw "D")
    , (chr 949, raw "e")
    , (chr 917, raw "E")
    , (chr 917, raw "E")
    , (chr 950, raw "z")
    , (chr 918, raw "Z")
    , (chr 918, raw "Z")
    , (chr 951, raw "h")
    , (chr 919, raw "H")
    , (chr 919, raw "H")
    , (chr 952, raw "j")
    , (chr 920, raw "J")
    , (chr 953, raw "i")
    , (chr 921, raw "I")
    , (chr 921, raw "I")
    , (chr 954, raw "k")
    , (chr 922, raw "K")
    , (chr 922, raw "K")
    , (chr 955, raw "l")
    , (chr 923, raw "L")
    , (chr 956, raw "m")
    , (chr 924, raw "M")
    , (chr 924, raw "M")
    , (chr 957, raw "n")
    , (chr 925, raw "N")
    , (chr 925, raw "N")
    , (chr 958, raw "x")
    , (chr 926, raw "X")
    , (chr 959, raw "o")
    , (chr 959, raw "o")
    , (chr 927, raw "O")
    , (chr 927, raw "O")
    , (chr 960, raw "p")
    , (chr 928, raw "P")
    , (chr 961, raw "r")
    , (chr 929, raw "P")
    , (chr 929, raw "R")
    , (chr 963, raw "s")
    , (chr 931, raw "S")
    , (chr 962, raw "c")
    , (chr 964, raw "t")
    , (chr 932, raw "T")
    , (chr 932, raw "T")
    , (chr 965, raw "u")
    , (chr 933, raw "Y")
    , (chr 933, raw "U")
    , (chr 966, raw "f")
    , (chr 934, raw "F")
    , (chr 967, raw "q")
    , (chr 935, raw "X")
    , (chr 935, raw "Q")
    , (chr 968, raw "y")
    , (chr 936, raw "Y")
    , (chr 969, raw "w")
    , (chr 937, raw "W")
    , (chr 940, raw "'a")
    , (chr 902, raw "'A")
    , (chr 941, raw "'e")
    , (chr 904, raw "'E")
    , (chr 942, raw "'h")
    , (chr 905, raw "'H")
    , (chr 970, raw "\"i")
    , (chr 938, raw "\"I")
    , (chr 943, raw "'i")
    , (chr 906, raw "'I")
    , (chr 912, raw "\"'i")
    , (chr 972, raw "'o")
    , (chr 908, raw "'O")
    , (chr 971, raw "\"u")
    , (chr 939, raw "\"U")
    , (chr 973, raw "'u")
    , (chr 910, raw "'U")
    , (chr 944, raw "\"'u")
    , (chr 974, raw "'w")
    , (chr 911, raw "'W")
    , (chr 8249, raw "\\guilsinglleft")
    , (chr 8250, raw "\\guilsinglright")
    , (chr 8254, raw "\\={}")
    , (chr 8260, raw "\\textfractionsolidus")
    , (chr 8364, raw "\\texteuro")
    , (chr 8672, m (raw "\\dashleftarrow"))
    , (chr 8674, m (raw "\\dashrightarrow"))
    , (chr 8674, m (raw "\\dasharrow"))
    , (chr 8487, raw "\\textmho")
    , (chr 8759, m (raw "::"))
    , (chr 12314, raw "\\textlbrackdbl")
    , (chr 8241, raw "\\textpertenthousand")
    , (chr 12315, raw "\\textrbrackdbl")
    , (chr 717, raw "\\b{}")
    , (chr 803, raw "\\d{}")
    , (chr 865, raw "\\t{}")
    , (chr 8251, raw "\\textreferencemark") ] ++
    latexEntities2

-- Extra mappings I added by hand.
latexEntities2 :: [(Char, LaTeX)]
latexEntities2 =
    [ (chr 146, raw "'")
    , (chr 146, raw "\\textquoteright") ]

-- | Entities for which we don't know the unicode codepoint
latexEntitiesUnknown :: [LaTeX]
latexEntitiesUnknown =
    [ m (raw "\\varepsilon")
    , m (raw "\\imath")
    , m (raw "\\jmath")
    , m (raw "{'}")
    , m (raw "\\smallsetminus")
    , m (raw "\\gtrapprox")
    , m (raw "\\geqslant")
    , m (raw "\\lessapprox")
    , m (raw "\\leqslant")
    , m (raw "\\precapprox")
    , m (raw "\\amalg")
    , m (raw "\\textstyle\\amalg")
    , m (raw "\\succapprox")
    , m (raw "\\smallfrown")
    , m (raw "\\shortmid")
    , m (raw "\\shortparallel")
    , m (raw "\\smallsmile")
    , m (raw "\\thickapprox")
    , m (raw "\\thicksim")
    , m (raw "\\gnapprox")
    , m (raw "\\lnapprox")
    , m (raw "\\ngeqq")
    , m (raw "\\nleqq")
    , m (raw "\\nshortmid")
    , m (raw "\\nshortparallel")
    , m (raw "\\precnapprox")
    , m (raw "\\precneqq")
    , m (raw "\\succnapprox")
    , m (raw "\\succneqq")
    , m (raw "\\varsubsetneqq")
    , m (raw "\\varsubsetneq")
    , m (raw "\\varsupsetneq")
    , m (raw "\\varsupsetneqq")
    , m (raw "\\Longleftarrow")
    , m (raw "\\Longleftrightarrow")
    , m (raw "\\longleftrightarrow")
    , m (raw "\\Longrightarrow")
    , m (raw "\\rlap{$>$}\\,)")
    , m (raw "\\rlap{$>$}\\,(")
    -- , m (raw "\\leftrightarrowtriangle")
    -- , m (raw "\\leftarrowtriangle")
    -- , m (raw "\\rightarrowtriangle")
    -- , m (raw "\\shortleftarrow")
    -- , m (raw "\\shortrightarrow")
    , m (raw "\\longleftarrow")
    , m (raw "\\longmapsto")
    , m (raw "\\longrightarrow")
    -- , m (raw "\\boxbslash")
    -- , m (raw "\\ogreaterthan")
    -- , m (raw "\\minuso")
    -- , m (raw "\\varocircle")
    -- , m (raw "\\olessthan")
    -- , m (raw "\\obar")
    -- , m (raw "\\boxslash")
    , m (raw "\\bigcap")
    , m (raw "\\bigcup")
    , m (raw "\\bigodot")
    , m (raw "\\bigoplus")
    , m (raw "\\bigotimes")
    , m (raw "\\bigsqcup")
    , m (raw "\\biguplus")
    , m (raw "\\bigvee")
    , m (raw "\\bigwedge")
    , m (raw "\\lmoustache")
    , m (raw "\\rlap{$<$}\\;(")
    , m (raw "\\rmoustache")
    , m (raw "\\varnothing")
    , m (raw "\\hslash")
    , m (raw "::=")
    , m (raw "><")
    , m (raw "\\rlap{$>$}\\!<")
    -- , m (raw "\\rightslice")
    -- , m (raw "\\leftslice")
    , m (raw "a")
    , m (raw "A")
    , m (raw "b")
    , m (raw "B")
    , m (raw "c")
    , m (raw "C")
    , m (raw "d")
    , m (raw "D")
    , m (raw "e")
    , m (raw "E")
    , m (raw "f")
    , m (raw "F")
    , m (raw "g")
    , m (raw "G")
    , m (raw "h")
    , m (raw "H")
    , m (raw "i")
    , m (raw "I")
    , m (raw "j")
    , m (raw "J")
    , m (raw "k")
    , m (raw "K")
    , m (raw "l")
    , m (raw "L")
    , m (raw "m")
    , m (raw "M")
    , m (raw "n")
    , m (raw "N")
    , m (raw "o")
    , m (raw "O")
    , m (raw "p")
    , m (raw "P")
    , m (raw "q")
    , m (raw "Q")
    , m (raw "r")
    , m (raw "R")
    , m (raw "s")
    , m (raw "S")
    , m (raw "t")
    , m (raw "T")
    , m (raw "u")
    , m (raw "U")
    , m (raw "v")
    , m (raw "V")
    , m (raw "w")
    , m (raw "W")
    , m (raw "x")
    , m (raw "X")
    , m (raw "y")
    , m (raw "Y")
    , m (raw "z")
    , m (raw "Z")
    , m (raw "A")
    , m (raw "B")
    , m (raw "C")
    , m (raw "D")
    , m (raw "E")
    , m (raw "F")
    , m (raw "G")
    , m (raw "H")
    , m (raw "I")
    , m (raw "J")
    , m (raw "K")
    , m (raw "L")
    , m (raw "M")
    , m (raw "N")
    , m (raw "O")
    , m (raw "P")
    , m (raw "Q")
    , m (raw "R")
    , m (raw "S")
    , m (raw "T")
    , m (raw "U")
    , m (raw "V")
    , m (raw "W")
    , m (raw "X")
    , m (raw "Y")
    , m (raw "Z")
    , m (raw "A")
    , m (raw "B")
    , m (raw "C")
    , m (raw "D")
    , m (raw "E")
    , m (raw "F")
    , m (raw "G")
    , m (raw "H")
    , m (raw "I")
    , m (raw "J")
    , m (raw "K")
    , m (raw "L")
    , m (raw "M")
    , m (raw "N")
    , m (raw "O")
    , m (raw "P")
    , m (raw "Q")
    , m (raw "R")
    , m (raw "S")
    , m (raw "T")
    , m (raw "U")
    , m (raw "V")
    , m (raw "W")
    , m (raw "X")
    , m (raw "Y")
    , m (raw "Z")
    , m (raw "\\AC")
    -- , m (raw "\\merge")
    , m (raw "\\cdots")
    , m (raw "\\ddots")
    , m (raw "\\int\\!\\!\\!\\int")
    -- , m (raw "\\inplus")
    -- , m (raw "\\niplus")
    , m (raw "\\int\\!\\!\\!\\int\\!\\!\\!\\int\\!\\!\\!\\int")
    -- , m (raw "\\rrparenthesis")
    , m (raw "\\int\\!\\!\\!\\int\\!\\!\\!\\int")
    , m (raw "\\!") ]


-- | Entities whose LaTeX definitions cause errors
latexEntitiesBroken :: [(Maybe Char, LaTeX)]
latexEntitiesBroken =
    [ (Just (chr 9617), raw "\\textcolor[gray]{.75}{\\block}")
    , (Just (chr 9618), raw "\\textcolor[gray]{.5}{\\block}")
    , (Just (chr 9619), raw "\\textcolor[gray]{.25}{\\block}")
    , (Just (chr 164), raw "\\currency")
    , (Just (chr 162), raw "\\cent")
    , (Just (chr 165), m (raw "\\yen"))
    , (Just (chr 183), m (raw "\\centerdot"))
    , (Just (chr 174), m (raw "\\circledR"))
    , (Just (chr 8735), m (raw "\\sqangle"))
    , (Just (chr 8240), raw "\\permil")
    , (Just (chr 8793), m (raw "\\wedgeq"))
    , (Just (chr 988), m (raw "\\digamma"))
    , (Just (chr 1008), m (raw "\\varkappa"))
    , (Just (chr 988), m (raw "\\digamma"))
    , (Just (chr 1008), m (raw "\\varkappa"))
    , (Just (chr 8502), m (raw "\\beth"))
    , (Just (chr 8245), m (raw "\\backprime"))
    , (Just (chr 8504), m (raw "\\daleth"))
    , (Just (chr 8503), m (raw "\\gimel"))
    , (Just (chr 8602), m (raw "\\nleftarrow"))
    , (Just (chr 8603), m (raw "\\nrightarrow"))
    , (Just (chr 240), m (raw "\\eth"))
    , (Just (chr 254), raw "\\thorn")
    , (Just (chr 222), raw "\\Thorn")
    , (Just (chr 728), m (raw "\\breve{}"))
    , (Just (chr 96), m (raw "\\grave{}"))
    , (Just (chr 1072), raw "\\cyra")
    , (Just (chr 1040), raw "\\CYRA")
    , (Just (chr 1073), raw "\\cyrb")
    , (Just (chr 1041), raw "\\CYRB")
    , (Just (chr 1074), raw "\\cyrv")
    , (Just (chr 1042), raw "\\CYRV")
    , (Just (chr 1075), raw "\\cyrg")
    , (Just (chr 1043), raw "\\CYRG")
    , (Just (chr 1076), raw "\\cyrd")
    , (Just (chr 1044), raw "\\CYRD")
    , (Just (chr 1077), raw "\\cyre")
    , (Just (chr 1045), raw "\\CYRE")
    , (Just (chr 1105), raw "\\cyryo")
    , (Just (chr 1025), raw "\\CYRYO")
    , (Just (chr 1078), raw "\\cyrzh")
    , (Just (chr 1046), raw "\\CYRZH")
    , (Just (chr 1079), raw "\\cyrz")
    , (Just (chr 1047), raw "\\CYRZ")
    , (Just (chr 1080), raw "\\cyri")
    , (Just (chr 1048), raw "\\CYRI")
    , (Just (chr 1081), raw "\\cyrishrt")
    , (Just (chr 1049), raw "\\CYRISHRT")
    , (Just (chr 1082), raw "\\cyrk")
    , (Just (chr 1050), raw "\\CYRK")
    , (Just (chr 1083), raw "\\cyrl")
    , (Just (chr 1051), raw "\\CYRL")
    , (Just (chr 1084), raw "\\cyrm")
    , (Just (chr 1052), raw "\\CYRM")
    , (Just (chr 1085), raw "\\cyrn")
    , (Just (chr 1053), raw "\\CYRN")
    , (Just (chr 1086), raw "\\cyro")
    , (Just (chr 1054), raw "\\CYRO")
    , (Just (chr 1087), raw "\\cyrp")
    , (Just (chr 1055), raw "\\CYRP")
    , (Just (chr 1088), raw "\\cyrr")
    , (Just (chr 1056), raw "\\CYRR")
    , (Just (chr 1089), raw "\\cyrs")
    , (Just (chr 1057), raw "\\CYRS")
    , (Just (chr 1090), raw "\\cyrt")
    , (Just (chr 1058), raw "\\CYRT")
    , (Just (chr 1091), raw "\\cyru")
    , (Just (chr 1059), raw "\\CYRU")
    , (Just (chr 1092), raw "\\cyrf")
    , (Just (chr 1060), raw "\\CYRF")
    , (Just (chr 1093), raw "\\cyrh")
    , (Just (chr 1061), raw "\\CYRH")
    , (Just (chr 1094), raw "\\cyrc")
    , (Just (chr 1062), raw "\\CYRC")
    , (Just (chr 1095), raw "\\cyrch")
    , (Just (chr 1063), raw "\\CYRCH")
    , (Just (chr 1096), raw "\\cyrsh")
    , (Just (chr 1064), raw "\\CYRSH")
    , (Just (chr 1097), raw "\\cyrshch")
    , (Just (chr 1065), raw "\\CYRSHCH")
    , (Just (chr 1098), raw "\\cdprime")
    , (Just (chr 1098), raw "\\cyrhrdsn")
    , (Just (chr 1066), raw "\\Cdprime")
    , (Just (chr 1066), raw "\\CYRHRDSN")
    , (Just (chr 1099), raw "\\cyrery")
    , (Just (chr 1067), raw "\\CYRERY")
    , (Just (chr 1100), raw "\\cprime")
    , (Just (chr 1100), raw "\\cyrsftsn")
    , (Just (chr 1068), raw "\\Cprime")
    , (Just (chr 1068), raw "\\CYRSFTSN")
    , (Just (chr 1101), raw "\\cyrerev")
    , (Just (chr 1069), raw "\\CYREREV")
    , (Just (chr 1102), raw "\\cyryu")
    , (Just (chr 1070), raw "\\CYRYU")
    , (Just (chr 1103), raw "\\cyrya")
    , (Just (chr 1071), raw "\\CYRYA")
    , (Just (chr 1106), raw "\\cyrdje")
    , (Just (chr 1026), raw "\\CYRDJE")
    , (Just (chr 1107), raw "\\cyrgup")
    , (Just (chr 1027), raw "\\CYRGUP")
    , (Just (chr 1108), raw "\\cyrie")
    , (Just (chr 1028), raw "\\CYRIE")
    , (Just (chr 1109), raw "\\dz")
    , (Just (chr 1109), raw "\\cyrdze")
    , (Just (chr 1029), raw "\\CYRDZE")
    , (Just (chr 1110), raw "\\cyrii")
    , (Just (chr 1030), raw "\\CYRII")
    , (Just (chr 1111), raw "\\cyryi")
    , (Just (chr 1031), raw "\\CYRYI")
    , (Just (chr 1112), raw "\\cyrje")
    , (Just (chr 1032), raw "\\CYRJE")
    , (Just (chr 1113), raw "\\cyrlje")
    , (Just (chr 1033), raw "\\CYRLJE")
    , (Just (chr 1114), raw "\\cyrnje")
    , (Just (chr 1034), raw "\\CYRNJE")
    , (Just (chr 1115), raw "\\cyrtshe")
    , (Just (chr 1035), raw "\\CYRTSHE")
    , (Just (chr 1116), raw "\\cyrkbeak")
    , (Just (chr 1036), raw "\\CYRKBEAK")
    , (Just (chr 1118), raw "\\cyrushrt")
    , (Just (chr 1038), raw "\\CYRUSHRT")
    , (Just (chr 1119), raw "\\cyrdzhe")
    , (Just (chr 1039), raw "\\CYRDZHE")
    , (Just (chr 9), raw "\\>,\\=")
    , (Just (chr 10), raw "\\\\")
    , (Just (chr 8487), m (raw "\\mho"))
    , (Just (chr 12314), m (raw "\\llbracket"))
    , (Just (chr 12315), m (raw "\\rrbracket")) ]

isoentHeaders :: LaTeXC l => l
isoentHeaders =
    usepackage [raw "utf8"] "inputenc" <> nl <>
    usepackage [raw "T1"] "fontenc" <> nl <>
    usepackage [] "longtable" <> nl <>
    usepackage [] "textcomp" <> nl <> -- textquotesingle, textquotedbl
    usepackage [] "amsmath" <> nl <>
    usepackage [] "amsbsy" <> nl <>
    usepackage [] "latexsym" <> nl <>
    usepackage [] "color" <> nl <>
    usepackage [] "amssymb" <> nl <>
    usepackage [] "wasysym" <> nl <>
    usepackage [] "pifont" <> nl <>
    raw isoentSty

nl :: LaTeXC l => l
nl = raw "\n"

m :: LaTeX -> LaTeX
m = TeXMath Dollar
